from hashlib import md5

from django.test import TestCase

from easymode.tree.serializers import RecursiveXmlSerializer

from easymode.tests.models import *
from easymode.tests.testcases import initdb
from easymode.utils.languagecode import get_language_codes
from easymode import tree

if 'en-us' not in get_language_codes():
    raise Exception('the language "en-us" must be in your LANGUAGES to run the test suite')

# CONFIRMED_XML_DIGEST = 'ffcb6719a199ffc189ae35bff10a29d0'
CONFIRMED_XML_DIGEST = '02658bdf2a2e131ae4027396672037f7'
@initdb
class RecursiveSerializerTest(TestCase):
    """
    Test the functionality related to
    xmlserializable models.
    """
    
    def extraSetUp(self):
        t = TestModel(charfield='In lectus est, viverra a, ultricies ut, pulvinar vitae, tellus.')
        t.save()
        f = t.submodels.create(subcharfield="Quisque facilisis erat a dui.", subintegerfield=10)
        s = t.submodels.create(subcharfield="Hoi ik ben de tweede first level sub node", subintegerfield=100)
        w = t.secondsubmodels.create(ultrafield="Sed tempor. Ut felis. Maecenas erat.")
        f.subsubmodels.create(subsubcharfield="Cras eu mauris.")
        f.subsubmodels.create(subsubcharfield="Etiam imperdiet urna sit amet risus.")
        s.subsubmodels.create(subsubcharfield="Proin tincidunt, velit vel porta elementum, magna diam molestie sapien, non aliquet massa pede eu diam.")
        s.subsubmodels.create(subsubcharfield="Ut felis.")        
        t.save()
    
    
    def test_create_test_models(self):
        """create test models"""
        assert(TestModel.objects.all().count() == 1)
        
    def test_tree_serialize(self):
        """test serialization of models with foreign keys"""
        ser = RecursiveXmlSerializer()
        res = TestModel.objects.all()
        xml = ser.serialize(res)
        assert(md5(xml).hexdigest() == CONFIRMED_XML_DIGEST)
        
    def test_model_to_xml(self):
        """test the __xml__ method added to the model"""
        xml = TestModel.objects.get(pk=1).__xml__()
        assert( md5(xml).hexdigest() == CONFIRMED_XML_DIGEST)
 
    def test_model_to_xml_with_xml_function(self):
        """test the xml function in package tree"""
        xml = tree.xml(TestModel.objects.get(pk=1))
        assert( md5(xml).hexdigest() == CONFIRMED_XML_DIGEST)
        
    def test_query_set_to_xml(self):
        """Test the __xml__ method on querysets"""
        xml = TestModel.objects.all().__xml__()
        assert( md5(xml).hexdigest() == CONFIRMED_XML_DIGEST)
    
    def test_generic_relations_also_work(self):
        """Generic relations should also be converted to xml properly"""
        r = TestGenericFkModel(name='ik ben cool')
        r.save()
        r.relateds.create(name='ik ben graaf')
        r.save()
        # print r
        # print "\n"
        # print r.__dict__
        # print "\n"
        # print r.__class__.__dict__
        # print "\n"
        # print r.__xml__()
        assert("This test must be implemented, it does work!" is False)


from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic

from easymode.tree.decorators import toxml
from easymode.admin.models.fields import *
from easymode.i18n.decorators import I18n

@I18n('charfield')
@toxml
class TestModel(models.Model):
    """test model"""

    charfield = models.CharField(max_length=255)
    
    def __unicode__(self):
        return u"%s%s - %s" % (self.__class__.__name__, self.pk, self.charfield)

class TestSubModel(models.Model):
    """hangs onto testmodel"""

    testmodel = models.ForeignKey(TestModel, related_name='submodels')
    subcharfield = models.CharField(max_length=255)
    subintegerfield = models.IntegerField()

    def __unicode__(self):
        return u"%s%s - %s" % (self.__class__.__name__, self.pk, self.subcharfield)
    
class TestSecondSubmodel(models.Model):
    """hangs on to TestModel as well"""
    
    testmodel = models.ForeignKey(TestModel, related_name='secondsubmodels')
    ultrafield = models.TextField()
    
    def __unicode__(self):
        return self.ultrafield

class TestSubSubModel(models.Model):
    """Even subier sub model"""
    
    testsubmodel = models.ForeignKey(TestSubModel, related_name="subsubmodels")
    subsubcharfield = models.CharField(max_length=255)
    
    def __unicode__(self):
        return self.subsubcharfield

@toxml
class UrlFieldTestModel(models.Model):
    title    = models.CharField(max_length=256)
    revision = models.PositiveIntegerField()
    url      = FlashUrlField(max_length=256)
# 
# @toxml
# class IncludeFileFieldTestModel(models.Model):
#     title = models.CharField(max_length=256)
#     file  = IncludeFileField(base=os.path.dirname(__file__), relative_path='hax')
# 
# @toxml
# class RemoteIncludeFieldModel(models.Model):
#     title = models.CharField(max_length=256)
#     url   = RemoteIncludeField(interval=60)

@toxml
@I18n('title', 'description')
class TestL10nModel(models.Model):

    title = DiocoreCharField(max_length=200, font="arial")
    description = DiocoreTextField(font="arial")
    body = DiocoreHTMLField(default='hi i am an error', font="arial")
    price = models.FloatField()

    def __unicode__(self):
        return u"%s%s" % (self.__class__.__name__, self.pk)

class GenericRelatedModel(models.Model):
    """it is used in a generic relation"""

    content_type = models.ForeignKey(ContentType)
    parent_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'parent_id')

    name = models.CharField(max_length=100)

    def __unicode__(self):
        return u"%s%s" % (self.__class__.__name__, self.pk)

@toxml
class TestGenericFkModel(models.Model):
    """it is used for testing"""

    name = models.CharField(max_length=100)
    relateds = generic.GenericRelation(GenericRelatedModel, content_type_field='content_type', object_id_field='parent_id')

    def __unicode__(self):
        return u"%s%s" % (self.__class__.__name__, self.pk)





from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.conf import settings

PUBLICATION_STATUSSES = (
    ('published', _('Published')),
    ('draft', _('Draft')),
    ('needs_work', _('Needs work')),
    ('updated', _('Updated')),
)

class EasyPublisherMetaData(models.Model):
    """
    A model for adding metadata to a reversion version.
    """
    
    class Meta:
        permissions = (('can_approve_for_publication', 'Can approve for publication'),)
    
    revision = models.ForeignKey("reversion.Revision")
    status = models.CharField(_('Publication Status'), max_length=10, choices=PUBLICATION_STATUSSES, default='draft')
    language = models.CharField(_('Language'), max_length=5, choices=settings.LANGUAGES, default=settings.LANGUAGE_CODE)
    
    def __unicode__(self):
        return u"revision: %s status: %s language: %s" % (self.revision, self.status, self.language)
    


from django.test import TestCase
from django.conf import settings

import re

from easymode.tree    import xml
from easymode.tests.models import UrlFieldTestModel
from easymode.tests.testcases import initdb

@initdb
class HtmlFlashUrlFieldTest(TestCase):
    def extraSetUp(self):
        settings.BASE_URL = '/virtuele-camping/'

        UrlFieldTestModel.objects.create( url     ='onderdeel/detail-pagina',
                                          title   ='Main Page',
                                          revision=7 )

    def test_serialization(self):
        """Test HtmlFlashUrlField serialization"""

        t = UrlFieldTestModel.objects.all()[0]

        x = xml(t)

        tm = re.search(r'<field[^>]*name="title"[^>]*>([^<]*)</field>', x)
        self.assertTrue(tm)
        self.assertEquals('Main Page', tm.group(1).strip())

        tr = re.search(r'<field[^>]*name="revision"[^>]*>([^<]*)</field>', x)
        self.assertTrue(tr)
        self.assertEquals('7', tr.group(1).strip())

        tu = re.search(r'<field[^>]*name="url"[^>]*>(.*?)</field>', x)
        self.assertTrue(tu)
        content = tu.group(1)

        hu = re.search(r'<url-html[^>]*value="([^"]*)"', content)
        self.assertTrue(hu)
        self.assertEquals( '/virtuele-camping/onderdeel/detail-pagina',
                           hu.group(1) )

        hf = re.search(r'<url-flash[^>]*value="([^"]*)"', content)
        self.assertTrue(hf)
        self.assertEquals( '/virtuele-camping/onderdeel/detail-pagina',
                           hf.group(1) )



from django import template
from django.core.urlresolvers import reverse
from django.utils.safestring import mark_safe

from reversion.models import Revision

from easymode.utils.languagecode import strip_language_code

register = template.Library()

#todo admin site is hardcoded, should get current admin name from somewhere
@register.tag
def draft_list_items(parser, token):
    """displays a list of all drafts, without the enclosing <ul>"""
    return DraftListItems()

class DraftListItems(template.Node):
    def render(self, context):
        request = context['request']

        #select revisions
        revisions = Revision.objects.filter(
            easypublishermetadata__status='draft',
            easypublishermetadata__language=request.LANGUAGE_CODE
        ).select_related().distinct()
    
        output = u''
        for revision in revisions:
            # select version and make list item
            version = revision.version_set.all().order_by('revision')[0]
            opts = version.content_type.model_class()._meta
            info = opts.app_label, opts.module_name
            name = 'admin:%s_%s_draft' % info
            url = reverse(name, args=[version.object_id, version.id])
            url = strip_language_code(url)
            output += u"<li><a href=\"%s\">%s</a></li>" % (url, version.object_repr)
    
        return mark_safe(output)



from StringIO import StringIO
import inspect

def stack_trace(depth=None):
    """returns a stack trace"""
    frames = inspect.stack()[2:]
    if depth:
        frames = frames[:depth]
        
    result = StringIO()
    result.write("----------------------------------------------------\n")
    for (frame, file, line, context, code, status) in frames:
        result.write("In %s from %s\n%s %s" % (context, file, line, "\n".join(code)))
        result.write("----------------------------------------------------\n")        
    return result.getvalue()




from django.contrib import admin

from easymode.tests.models import TestSubModel, TestModel, TestSecondSubmodel, TestL10nModel
from easymode.tree.admin import relation

from easymode.i18n.admin.decorators import L10n
from easymode.easypublisher.admin import EasyPublisher

class TestSubModelInline(admin.StackedInline):
# class TestSubModelInline(inline.RecursiveInline):
# class TestSubModelInline(admin.SelectorInline):
    """docstring for TestSubModelInline"""
    model = TestSubModel
    extra = 1
    order_field = 'order'

        
class TestModelAdmin(relation.ForeignKeyAwareModelAdmin):
# class TestModelAdmin(admin.ModelAdmin):
    """docstring for TestModelAdmin"""
    # fieldsets = [
    #     ('main name', {'fields': ['charfield'], 'classes': ['collapse']})
    # ]
    inlines = [TestSubModelInline]
    
    children = [TestSubModel,TestSecondSubmodel]
    invisible_in_admin = False
    
class SelfAwareTestModelAdmin(relation.ForeignKeyAwareModelAdmin):
    model = TestModel

@L10n(TestL10nModel)
class TestL10nModelAdmin(EasyPublisher):
    pass


admin.site.register(TestModel, TestModelAdmin)
admin.site.register(TestSubModel)
admin.site.register(TestSecondSubmodel)
admin.site.register(TestL10nModel, TestL10nModelAdmin)



import glob
import re
import os

from django.conf import settings

"""
Imports all testcases found in the 'testcases' subfolder of 
folder containg the file parameter.
Doing that is enough to make the testrunner find them.

so 
>>> import_testcases("/tmp/stuff/tests.py")
will import all testcases in /tmp/stuff/testcases/
"""

path = os.path.join(os.path.dirname(__file__), 'testcases')
pattern = os.path.join(path , r'(.+).py')

filenames = glob.glob(path + '/*.py')

if len(filenames): #delete __init__.py. yes it is always first.
    del(filenames[0])

from easymode.tests.testcases.testdiocorefields import *
from easymode.tests.testcases.testeasypublisher import *
from easymode.tests.testcases.testi18n import *
from easymode.tests.testcases.testrelatedadmin import *
from easymode.tests.testcases.testtoxml import *
from easymode.tests.testcases.testxslt import *

from easymode.utils.languagecode import *
__test__ = {
    "test_get_language_codes" : get_language_codes,
    "test_get_short_language_codes" : get_short_language_codes,
    "test_get_language_code_from_shorthand" : get_language_code_from_shorthand,
    "test_strip_language_code" : strip_language_code,
    "test_get_real_fieldname" : get_real_fieldname,
    "test_localize_fieldnames" : localize_fieldnames,
    "get_language_codes_as_disjunction" : get_language_codes_as_disjunction,
}
#for filename in filenames: # import testcases into current scoope
#    modulename = re.sub(pattern, r'\1', filename)
#    try:
#        exec("from easymode.tests.testcases.%s import *" % modulename)
#    except Exception, e:
#        print "%s in %s, please correct if you want to run this test suite." % (e, modulename)



"""
functionality for finding inverse foreign key relations in model classes
"""
import inspect

from django.contrib.contenttypes.generic import ReverseGenericRelatedObjectsDescriptor
from django.db.models.fields.related import ForeignRelatedObjectsDescriptor, SingleRelatedObjectDescriptor
from django.db.models.base import ModelBase

from easymode.i18n.meta import DefaultFieldDescriptor

SUBFIELDBASE_ERROR = """
%s

Easymode caught an AttributeError while trying
to inspect %s looking for %s. This is probably caused
by SubFieldBase, see http://code.djangoproject.com/ticket/12568.
Easymode will patch SubFieldBase for you, you just have to make sure
not to reference it before easymode is imported. This
means you can not load any apps that use SubFieldBase before
you load easymode.
"""

def _get_members_of_type(obj, member_type):
    """
    Finds members of a certain type in obj.

    :param obj: A model instance or class.
    :param member_type: The type of the menber we are trying to find.
    :rtype: A :class:`list` of ``member_type`` found in ``obj``
    """
    try:
        def filter_member_type(member):
            try:
                return type(member) is member_type
            except AttributeError:
                return False
                
        # if the type of obj is the metaclass for all models, just search in the object
        # because it is not a model instance but a type
        if type(obj) is ModelBase:
            key_hash = inspect.getmembers(obj, filter_member_type)
        else:
            key_hash = inspect.getmembers(obj.__class__, filter_member_type)
            
    except AttributeError as e:
        raise AttributeError(SUBFIELDBASE_ERROR % (e, obj, member_type))

    return key_hash

def get_foreign_key_desciptors(obj):
    """
    finds all :class:`~django.db.models.fields.ForeignRelatedObjectsDescriptor` in obj.
    
    :param obj: A model instance or class.
    """
    return _get_members_of_type(obj, ForeignRelatedObjectsDescriptor)

def get_one_to_one_descriptors(obj):
    """
    finds all :class:`~django.db.models.fields.SingleRelatedObjectDescriptor` in obj.

    :param obj: A model instance or class.
    """
    return _get_members_of_type(obj, SingleRelatedObjectDescriptor)

def get_generic_relation_descriptors(obj):
    """
    Finds all the :class:`~django.contrib.contenttypes.generic.ReverseGenericRelatedObjectsDescriptor` in obj.
    
    :param obj: A model instance or class.
    """
    return _get_members_of_type(obj, ReverseGenericRelatedObjectsDescriptor)

def get_default_field_descriptors(obj):
    """
    find all  :class:`~easymode.i18n.meta.DefaultFieldDescriptor` in obj.
    
    :param obj: A model instance or class.
    """
    return _get_members_of_type(obj, DefaultFieldDescriptor)



# -*- coding: utf-8 -*-
import warnings
import re
import shutil
import os
import errno
import gc
import time

import gettext as gettext_module
from hashlib import sha1

from os.path import join, isdir


from django.test import TestCase
from django.db.models.signals import post_save
from django.conf import settings
from django.utils import translation
from django.utils.translation import trans_real

if 'rosetta' in settings.INSTALLED_APPS:
    from rosetta import polib
else:
    import polib

from easymode.utils import first_match, mutex, SemaphoreException
from easymode.utils.languagecode import get_language_codes
from easymode.i18n import meta
from easymode.tests.testcases import initdb
from easymode.tests import models
from easymode.i18n.gettext import MakeModelMessages
from easymode.i18n import gettext
from easymode.tests.testutils.scriptutil import ffindgrep
from easymode import tree

if 'de' not in get_language_codes():
    raise Exception('the language "de" must be in your LANGUAGES to run the test suite')
if 'en-us' not in get_language_codes():
    raise Exception('the language "en-us" must be in your LANGUAGES to run the test suite')

if settings.LANGUAGE_CODE is not 'en':
    raise Exception('To run the test suite the LANGUAGE_CODE must be set to "en"')

@initdb
class Testi18n(TestCase):
    """tests for the internationalisation/localisation support"""

    def extraSetUp(self):        
        gc.collect()
        translation.activate(settings.LANGUAGE_CODE)
        
        try:
            t = models.TestModel(charfield='Hoi Ik ben de root node')
            t.save()
            f = t.submodels.create(subcharfield="Hoi Ik ben de first level sub node", subintegerfield=10)
            s = t.submodels.create(subcharfield="Hoi ik ben de tweede first level sub node", subintegerfield=100)
            w = t.secondsubmodels.create(ultrafield="Sed tempor. Ut felis. Maecenas erat.")
            f.subsubmodels.create(subsubcharfield="Hoi ik ben de thord level sub node")
            f.subsubmodels.create(subsubcharfield="Hoi ik ben de third level sub node")
            s.subsubmodels.create(subsubcharfield="Hoi ik ben de third level sub node")
            s.subsubmodels.create(subsubcharfield="Hoi ik ben de third level sub node")        
            t.save()            
        except OSError as e:
            if e.errno == 24:
                print "hai er zit een bug in de mutex"

    # some vars
    title_en = 'HI I GUESS I AM THE TITLE IN ENGLISH'
    title_de = 'ICH LIEBE DICH'

    arr = [None, None, None, 'hai', 'True', True, 'last']
    
    def setup_l10n_model(self):
        translation.activate('en')
        i = models.TestL10nModel(title="Ik ben de groot moeftie van cambodja", 
            description="Ik weet weinig tot niks van brei cursussen", 
            body="Het zou kunnen dat linksaf slaan hier een goede keuze is.",
            price=12.0,
            )
        i.save()
        return i
    
    def setup_mofile_with_entry(self, poentry, language='en'):
        locale_location = join(settings.LOCALE_DIR, 'locale', language, 'LC_MESSAGES')
        pofile = polib.pofile(join(locale_location, 'django.po'))

        pofile.append(poentry)
        pofile.save()
        pofile.save_as_mofile(join(locale_location, 'django.mo'))
        
        jit_locale = gettext_module.translation('django', join(settings.LOCALE_DIR, 'locale'), [trans_real.to_locale(language)], trans_real.DjangoTranslation)
        jit_locale.set_language(language)
        
        locale = trans_real.translation(language)
        locale.merge(jit_locale)
        
    
    def test_locale_dir_is_not_project_dir(self):
        """The locale dir should be somewhere safe"""
        assert(not (os.path.normpath(settings.LOCALE_DIR) == os.path.normpath(settings.PROJECT_DIR)))
        
    def test_save_signal_is_caught(self):
        """when a model is saved a signal should be caught"""

        def stub(sender, **kwargs):
            self.save_was_caught = True
            self.save_was_a_created = kwargs['created']

        self.save_was_caught = False
            
        post_save.connect(stub, models.TestModel)
        i = models.TestModel.objects.get(pk=1)
        i.charfield = 'hai mom'
        i.save()
        assert(self.save_was_caught)
        
    def test_created_property_is_set(self):
        """A signal sent when a model is created should have the created property set to true"""
        def stub(sender, **kwargs):
            self.save_was_caught = True
            self.save_was_a_created = kwargs['created']

        self.save_was_caught = False
        self.save_was_a_created = False
        
        post_save.connect(stub, models.TestModel)
        i = models.TestModel(charfield='hai mother')
        i.save()
        assert(self.save_was_caught)
        assert(self.save_was_a_created)

    def test_we_have_the_proper_version_of_gettext(self):
        """The proper version of xgettext (>= 0.15) should be installed on the server."""
        
        if gettext.XGETTEXT_REENCODES_UTF8 is True:
            warnings.warn('while not fatal it is better to install a version of gettext \
                higher or equal to 0.15')
                    
    def test_model_can_be_serialized(self):
        """A model should be serializable to po"""
        db_gettext = MakeModelMessages()
        i = models.TestModel.objects.get(pk=1)

        m = str(db_gettext.poify(i))

        assert(m.index('msgid "Hoi Ik ben de root node"') is not None)
    
    def test_model_to_po(self):
        """A newly created model should be serialized to po"""
        locale_dir = join(settings.LOCALE_DIR, 'locale')
        if isdir(locale_dir):
            shutil.rmtree(locale_dir)
            
        assert(not isdir(locale_dir))
        i = models.TestModel(charfield='ik word serialized to po')
        i.save()
        assert(isdir(locale_dir))
        
        result = ffindgrep(locale_dir, ['^msgid "ik word serialized to po"$'])
        assert(result != {})
        
    def test_meta(self):
        """Transmeta should have a title_en as well as a title is default locale is en"""
        i = self.setup_l10n_model()
        
        assert(i.title_en == i.title == 'Ik ben de groot moeftie van cambodja')
    
    def test_first_true(self):

        assert(first_match(lambda x: x or None, self.arr))

    def test_meta_field_finder(self):
        """The correct localised version of the title property should be found"""
        field = meta.get_localized_property(self, 'title')
        assert(field == self.title_en)
        translation.activate('de')
        field = meta.get_localized_property(self, 'title')
        assert(field == self.title_de)

    def test_meta_get_localized_field_name(self):
        """Transmeta's get_localized_field_name should return the correct localized property name"""
        field = meta.get_localized_field_name(self, 'title')
        assert(field == 'title_en')
            
    def test_meta_now_selects_correct_field_on_propert_write(self):
        """Transmeta should write to title_en if language is en and property is title"""            
        i = self.setup_l10n_model()
        i.title = 'ik wijzig em ff ok?'
        i.save()
        assert(i.title == i.title_en == 'ik wijzig em ff ok?')
    
    def test_ugettext_lazy(self):
        """ugettext_lazy should return translated strings, if they are in the locale"""

        self.setup_l10n_model()
        
        poentry = polib.POEntry(msgid='test string', msgstr='LOL DID YOU REALLY THINK I WAS NUB?')
        
        self.setup_mofile_with_entry(poentry)
        
        lol = translation.ugettext_lazy("test string")
        assert(unicode(lol) == u'LOL DID YOU REALLY THINK I WAS NUB?')
    
    def test_adding_l10n_decorator_means_model_gets_rendered_to_po_file(self):
        """If a model is created, which is decorated with l10n, it should be added to the po file"""
        self.setup_l10n_model()

        result = ffindgrep(join(settings.LOCALE_DIR, 'locale'), 
            ['^msgid "Ik ben de groot moeftie van cambodja"$'])
        assert(result != {})
        
    def test_nulled_translations_get_translated_from_po_file(self):
        """If a property which is a translation is accessed but it is None, the translation from the po file should be used"""
        i = self.setup_l10n_model()
        
        assert(i.title == i.title_en == u'Ik ben de groot moeftie van cambodja')
        
        poentry = polib.POEntry(msgid=u'Ik ben de groot moeftie van cambodja', msgstr=u'Ik ben een zwerver')        
        self.setup_mofile_with_entry(poentry, 'de')
        
        translation.activate('de')

        assert(i.title_de is None)
        assert(i.title == u'Ik ben een zwerver')
    
    def test_translations_that_are_not_null_should_not_come_from_the_po_file(self):
        """If a translation is not null, that value should be returned."""
        i = self.setup_l10n_model()
        i.title_de = u'Ik ben een konijntje'
        i.save()
        
        assert(i.title == i.title_en == u'Ik ben de groot moeftie van cambodja')
        poentry = polib.POEntry(msgid=u'Ik ben de groot moeftie van cambodja', msgstr=u'Ik ben een zwerver')        
        self.setup_mofile_with_entry(poentry, 'de')
        
        translation.activate('de')

        assert(i.title_de is not None)
        assert(not (i.title == u'Ik ben een zwerver'))
        assert(i.title == u'Ik ben een konijntje')
        
    def test_translated_fields_handle_correctly_under_to_xml(self):
        """A field that is translated should show the correct value when converted to xml"""

        i = self.setup_l10n_model()

        poentry = polib.POEntry(msgid=u'Ik ben de groot moeftie van cambodja', msgstr=u'Ik ben een zwerver')        
        self.setup_mofile_with_entry(poentry, 'de')

        xml_representation = tree.xml(i)

        # todo:
        # DefaultFieldDescriptor is now serialized as a charfield type.
        # add the different serializable types to the xslt and the test should be able
        # to verify the correct workings then.
        # contains_default_field_descriptor = re.search(r'DefaultFieldDescriptor', xml_representation)
        # self.assertTrue(contains_default_field_descriptor)
        
        contains_moeftie = re.search(r'Ik ben de groot moeftie van cambodja', xml_representation)
        contains_konijntje = contains_konijntje = re.search(r'Ik ben een konijntje', xml_representation)        
        self.assertTrue(contains_moeftie)
        self.assertFalse(contains_konijntje)

        translation.activate('de')
        i.title = u'Ik ben een duits konijntje'
        xml_representation = tree.xml(i)
        contains_konijntje = re.search(r'Ik ben een duits konijntje', xml_representation)
        self.assertTrue(contains_konijntje)

        i.title_de = u'Ik ben geen konijntje'
        i.save()

        xml_representation = tree.xml(i)
        contains_konijntje = re.search(r'Ik ben geen konijntje', xml_representation)
        self.assertTrue(contains_konijntje)
                
    def test_semaphore(self):
        """the semaphore should bind a file to the context and hold on to it untill after the context is closed"""
        lockfile = join('/tmp', sha1(settings.SECRET_KEY).hexdigest() +'.semaphore')
        for a in range(1, 4):
            with mutex():
                assert(os.path.isfile(lockfile))
        
            assert(not os.path.exists(lockfile))

    def test_semaphore_exception(self):
        """The semaphore should throw an exception when the wait time is over"""
        lockfile = join('/tmp', sha1(settings.SECRET_KEY).hexdigest() +'.semaphore')
        try:
            with mutex(max_wait=0, lockfile=lockfile):
                with mutex(max_wait=0, lockfile=lockfile):
                    pass
        except SemaphoreException as e:
            pass
        else:
            self.fail('Should have triggered SemaphoreException')
        self.assertFalse(os.path.exists(lockfile))
        
    def test_fdopen_works_correctly(self):
        """fdopen should allow proper locking"""
        lockfile = join('/tmp', sha1(settings.SECRET_KEY).hexdigest() +'.semaphore')
        
        try:
            ty = os.open(lockfile, os.O_EXCL | os.O_RDWR | os.O_CREAT)
            fd = os.open(lockfile, os.O_EXCL | os.O_RDWR | os.O_CREAT)
        except OSError as e:
            self.assertEqual(e.errno, errno.EEXIST)
        else:
            self.fail("If the file exists an OSError should be thrown")
        finally:
            os.remove(lockfile)


from django.utils.safestring import mark_safe
from django.contrib.admin.widgets import RelatedFieldWidgetWrapper

class WidgetWrapper(RelatedFieldWidgetWrapper):
    """
    This class is a wrapper to a given widget to add a marker next to
    the field, so you can see that the field is translated.
    """
    def __init__(self, widget, **kwargs):            
        if type(widget) is WidgetWrapper:
            self.widget = widget.widget
        else:
            self.widget = widget
        
        self.is_hidden = self.widget.is_hidden
        self.needs_multipart_form = self.widget.needs_multipart_form
        self.attrs = self.widget.attrs
        self.choices = getattr(self.widget, 'choices', None)
    
    def render(self, name, value, *args, **kwargs):
        widget_html = self.widget.render(name, value, *args, **kwargs)
        return mark_safe(u'<div class="localized">%s <small>%s</small></div>' % (widget_html, unichr(8756)))



from django.contrib import admin
from django.db.models.base import ModelBase

from easymode.i18n.admin.decorators import L10n

__all__ = ('register_all',)

def register_all(module):
    """
    Register all models in _module_ with the admin
    
    :param module: The module that contains the models
    """
    member_names = dir(module)

    for member_name in member_names:
        obj = getattr(module, member_name, None)
        if isinstance(obj, ModelBase):
            if hasattr(obj, 'localized_fields'):
                admin.site.register(obj, L10n(obj, admin.ModelAdmin))
            else:
                admin.site.register(obj)



"""
Easymode is an aspect oriented toolkit that helps making xml based flash websites easy.
The tools included in the toolkit to help you make these kind of sites include:

* Internationalization of models, with admin support
* Translation of model data using gettext
* Automatic generation of xml from model trees using xslt
* Admin support for model trees with more than 2 levels of related items
* Basic approval support for models

documentation at http://packages.python.org/django-easymode/

release notes at http://packages.python.org/django-easymode/changes.html
"""

# monkey patch a bug in django's SubfieldBase
# See easymode.hacks.SubfieldBaseFix 
from easymode.hacks import SubfieldBaseFix
import django.db.models.fields.subclassing
django.db.models.fields.subclassing.SubfieldBase = SubfieldBaseFix
import django.db.models
django.db.models.SubfieldBase = SubfieldBaseFix



"""
Generic django admin extensions.
"""

from django.contrib import admin

class DragAndDropAdmin(admin.ModelAdmin):
    """
    Admin class that allows dragging and dropping of items
    in the admin listing.
    """
    ordering = ('order',)
    list_display = ('id', 'order')
    list_editable = ('order',)
    
    class Media:
        js = (
            'js/lib/jquery.js',
            'js/lib/ui.core.js',
            'js/lib/ui.sortable.js',
            'easymode/js/menu-sort.js',            
        )
        
        



from django.db import models

class AbstractOrderedModel(models.Model):    
    """
    This model includes functionality for ordering.
    """
    order = models.AutoField()
            
    class Meta:
        abstract = True
        ordering = ('order',)
    


    

        


#!/usr/bin/env python
import os
from distutils.core import setup
from distutils.command.install import INSTALL_SCHEMES

description="""Easymode is an aspect oriented toolkit that helps making xml based flash websites easy.
The tools included in the toolkit to help you make these kind of sites include:

* Internationalization of models, with admin support
* Translation of model data using gettext
* Automatic generation of xml from model trees using xslt
* Admin support for model trees with more than 2 levels of related items
* Basic approval support for models

documentation at http://packages.python.org/django-easymode/

release notes at http://packages.python.org/django-easymode/changes.html"""

version = '0.9.1'
packages = []
data_files = []

for scheme in INSTALL_SCHEMES.values():
    scheme["data"] = scheme["purelib"]

def fullsplit(path, result=None):
    """
    Split a pathname into components (the opposite of os.path.join) in a
    platform-neutral way.
    """
    if result is None:
        result = []
    head, tail = os.path.split(path)
    if head == '':
        return [tail] + result
    if head == path:
        return result
    return fullsplit(head, [tail] + result)

for dirpath, dirnames, filenames in os.walk('easymode'):
    # Ignore dirnames that start with '.'
    for i, dirname in enumerate(dirnames):
        if dirname.startswith('.'): del dirnames[i]
    if '__init__.py' in filenames:
        packages.append('.'.join(fullsplit(dirpath)))
    elif filenames:
        data_files.append([dirpath, [os.path.join(dirpath, f) for f in filenames]])

setup(name='django-easymode',
    version=version,
    description='Toolkit to ease development of internationalized xml based (flash) websites with django',
    author='L. van de Kerkhof',
    maintainer='L. van de Kerkhof',
    maintainer_email='easymode@librelist.com',
    long_description=description,
    url='http://github.com/LUKKIEN/django-easymode',
    packages=packages,
    data_files=data_files,
    platforms = "any",
    install_requires=['Django>=1.2.1', 'lxml>=2.2.2', 'polib>=0.5.1','django-reversion>=1.2.1'],
    license='GPL',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: Site Management',
        'Topic :: Software Development :: Localization',
    ],
)



"""
Derived from django-transmeta.
Contains decorator which make models internationalized.

This in contrast with django-transmeta which requires
overriding the __metaclass__.
"""
import copy

from django.conf              import settings
from django.contrib.admin.options import FORMFIELD_FOR_DBFIELD_DEFAULTS
from django.db.models.fields  import NOT_PROVIDED
from django.utils             import translation
from django.utils.encoding import force_unicode
from django.utils.translation.trans_real import translation as translation_catalogs

from easymode.utils           import first_match 
from easymode.utils.languagecode import get_all_language_codes, get_real_fieldname


def get_fallback_languages():
    """Retrieve the fallback languages from the settings.py"""
    lang = translation.get_language()
    fallback_list = settings.FALLBACK_LANGUAGES.get(lang, None)
    if fallback_list:
        return fallback_list

    return settings.FALLBACK_LANGUAGES.get(lang[:2], [])

def canonical_fieldname(db_field):
    """ all "description_en", "description_fr", etc. field names will return "description" """
    return getattr(db_field, 'original_fieldname', db_field.name) # original_fieldname is set by meta


def get_localized_property(context, field=None, language=None):
    '''
    When accessing to the name of the field itself, the value
    in the current language will be returned. Unless it's set,
    the value in the default language will be returned.
    '''
    if language:
        return getattr(context, get_real_fieldname(field, language))
    
    if settings.FALLBACK_LANGUAGES:
        attrs = [translation.get_language()]
        attrs += get_fallback_languages()
    else:
        attrs = [
            translation.get_language(),
            translation.get_language()[:2],
            settings.LANGUAGE_CODE, 
        ]
        
    predicate = lambda x: getattr(context, get_real_fieldname(field, x), None)

    return first_match(predicate, attrs)


def get_localized_field_name(context, field):
    """Get the name of the localized field"""
    attrs = [
             translation.get_language(), 
             translation.get_language()[:2], 
             settings.LANGUAGE_CODE
            ]
            
    def predicate(x):
        field_name = get_real_fieldname(field, x)
        if hasattr(context, field_name):
            return field_name
        return None

    return first_match(predicate, attrs)


class DefaultFieldDescriptor(property):
    """
    Descriptor that implements access to the default language.
    It inherits property because django wants that, it doesn't process
    it as a field if it is not isinstance(self, property)
    """

    def __init__(self, name, **kwargs):
        """The name param is the name of the field the descriptor is emulating."""
        self.name = name
        self.__dict__.update(kwargs)

    def __get__(self, obj, typ=None):
        """
        Read the localised version of the field this descriptor emulates.
        First try to see if the localised field is really set.
        If not, then use ugettext_lazy to find a tranlation in the current language
        for this field.
        """
        # self must be returned in a getattr context.
        if obj is None:
            return self
        
        current_language = translation.get_language()
        real_field_name = get_real_fieldname(self.name, current_language)
        
        # first check if the database contains the localized data
        stored_value = getattr(obj, real_field_name)
        if stored_value is not None:
            # if it is set, this is the value we are looking for
            return stored_value
        
        # the database does not have our localized data.
        # check if we have a translation, first get the msgid
        msgid_language = getattr(settings, 'MSGID_LANGUAGE', settings.LANGUAGE_CODE)
        msgid = force_unicode(get_localized_property(obj, self.name, msgid_language))
        
        if msgid:
            # check the translation in the current language
            msg = translation.ugettext(msgid)
            if self.to_python(msg) is not msgid:
                return self.to_python(msg)

        # maybe we have a translation in any of the fallback languages.
        if hasattr(settings, 'FALLBACK_LANGUAGES'):
            # first check if the database has the localized data in
            # any of the fallback languages.
            localized_value = get_localized_property(obj, self.name)

            if localized_value is not None:
                return localized_value
                
            # if the msgid is '' or None we don't have to look
            # for translations because there are none.
            if msgid in (None, ""):
                return self.to_python(u'')

            # there might be a translation in any
            # of the fallback languages.
            for fallback in get_fallback_languages():
                catalog = translation_catalogs(fallback)
                msg = catalog.ugettext(msgid)
                if self.to_python(msg) is not msgid:
                    return self.to_python(msg)
        
        # no fallback language and the database does not have
        # the localized data, so use the translation of the msgid
        if msgid in (None, ""):
            return self.to_python(u'')
        
        return self.to_python(msgid)

    def __set__(self, obj, value):
        """Write the localised version of the field this descriptor emulates."""
        local_property = get_localized_field_name(obj, self.name)
        setattr(obj, local_property, value)

    def get_internal_type(self):
        """Used to describe the type while serializing."""
        return "CharField"

    def value_to_string(self, obj):
        """This descriptor acts as a Field, as far as the serializer is concerned."""
        return  force_unicode(self.__get__(obj))


def localize_fields(cls, localized_fields):
    """
    For each field name in localized_fields,
    for each language in settings.LANGUAGES,
    add fields to cls,
    and remove the original field, instead
    replace it with a DefaultFieldDescriptor,
    which always returns the field in the current language.
    """
    
    # never do this twice
    if hasattr(cls, 'localized_fields'):
        return cls
        
    # MSGID_LANGUAGE is the language that is used for the gettext message id's.
    # If it is not available, because the site isn't using subsites, the LANGUAGE_CODE
    # is good too. MSGID_LANGUAGE gives the opportunity to specify a language not available
    # in the site but which is still used for the message id's.
    msgid_language = getattr(settings, 'MSGID_LANGUAGE',  settings.LANGUAGE_CODE)
    
    # set the localized fields property
    cls.localized_fields = localized_fields
    
    for field in localized_fields:
        original_attr = cls._meta.get_field_by_name(field)[0]
        
        for cnt, lang_code in enumerate(get_all_language_codes()):
            lang_attr = copy.copy(original_attr)
            # add support for south introspection.
            lang_attr._south_introspects = True
            lang_attr.original_fieldname = field
            lang_attr.include_in_xml = False
            lang_attr_name = get_real_fieldname(field, lang_code)
            lang_attr.creation_counter = lang_attr.creation_counter + .01 * cnt
            # null must be allowed for the message id language because this language
            # might not be available at all in the backend
            if not lang_attr.null and lang_attr.default is NOT_PROVIDED:
                lang_attr.null = True
            
            if lang_code != msgid_language:
                # no validation for the fields that are language specific
                if not lang_attr.blank:
                    lang_attr.blank = True
                    
            if lang_attr.verbose_name:
                lang_attr.verbose_name = translation.string_concat(lang_attr.verbose_name, u' (%s)' % lang_code)
            cls.add_to_class(lang_attr_name, lang_attr)
            
        # delete original field
        del cls._meta.local_fields[cls._meta.local_fields.index(original_attr)]

        # copy some values and functions from the original_attr
        # so the field can emulate the original_attr as good as possible
        kwargs = {
            'serialize': getattr(original_attr, 'serialize', True),
            'extra_attrs':getattr(original_attr, 'extra_attrs', None),
            'max_length': getattr(original_attr, 'max_length', None),
            'min_length' : getattr(original_attr, 'min_length', None),
            'form_field' : original_attr.formfield(**FORMFIELD_FOR_DBFIELD_DEFAULTS.get(original_attr.__class__, {})),
            'get_internal_type': original_attr.get_internal_type,
            'unique': getattr(original_attr, 'unique', False),
            'to_python': original_attr.to_python,
        }
            
        # copy custom_value_serializer if it was defined on the original attr
        if hasattr(original_attr, 'custom_value_serializer'):
            kwargs['custom_value_serializer'] = original_attr.custom_value_serializer

        # add the DefaultFieldDescriptor where the original_attr was.
        cls.add_to_class(field, DefaultFieldDescriptor(field, **kwargs))
        
        # update fields cache
        cls._meta._fill_fields_cache()

    # return the finished product
    return cls


# CMS Pages translation
def localize_cms_stuff_fields(cls, *localized_fields):
    cls.localized_fields = localized_fields
    return cls




import os.path
import shutil

from django.contrib.auth.models import User
from django.conf import settings

from easymode.tests.models import *

def initdb(cls):
    """
    This is a class decorator.
    
    Adds a setUp and tearDown method to a TestCase class.
    The setup will add tests to the installed apps, causing
    the models in the tests folder to be loaded.
    
    Then at the end they are removed again.
    """
    from easymode.tests.testutils import TestSettingsManager
    
    # return cls
    mgr = TestSettingsManager()
    
    def setUp(self):
        mgr.set(INSTALLED_APPS=settings.INSTALLED_APPS+[
            'easymode',
            'easymode.tests',
            ],
        )
        
        u = User(username="admin")
        u.set_password("admin")
        u.is_staff = True
        u.is_active = True
        u.is_superuser = True 
        u.save()
        
        for skipped_test in getattr(settings , 'SKIPPED_TESTS', []):
            if hasattr(cls, skipped_test):
                setattr(cls, skipped_test, lambda x: True)
            
        if hasattr(self, 'extraSetUp'):
            self.extraSetUp()
    
    def  tearDown(self):
        # don't try to remove the locale dir in the project dir because it has all translations
        # test cases should have a separate locale dir, preferably in /tmp
        if not (os.path.normpath(settings.LOCALE_DIR) == os.path.normpath(settings.PROJECT_DIR)):
            locale_dir = os.path.join(settings.LOCALE_DIR, 'locale')
            if os.path.exists(locale_dir):
                shutil.rmtree(locale_dir)
        mgr.revert()
        
    cls.setUp = setUp
    cls.tearDown = tearDown
    return cls



"""
Overrides django's subclassing.py because it breaks everything when trying to inspect
the model objects with inspect. This is because subclassing.Creator throws an exception
when it is accesses with no context. It SHOULD return the descriptor object
instead. And now it does.
"""
from django.db.models.fields.subclassing import LegacyConnection

class SubfieldBaseFix(LegacyConnection):
    """
    A metaclass for custom Field subclasses. This ensures the model's attribute
    has the descriptor protocol attached to it.

    MONKEYPATCHED by easymode
    
    The original SubfieldBase throws an exception when it's descriptor is accesses 
    with no context. It SHOULD return the descriptor object instead. And now it does.
    """
    def __new__(cls, base, name, attrs):
        new_class = super(SubfieldBaseFix, cls).__new__(cls, base, name, attrs)
        new_class.contribute_to_class = make_contrib(
                attrs.get('contribute_to_class'))
        return new_class

class CreatorFix(object):
    """
    A placeholder class that provides a way to set the attribute on the model.
    
    MONKEYPATCHED by easymode
    
    The original Creator throws an exception when it is accesses with no 
    context. It SHOULD return the descriptor object instead. And now it does.
    """
    def __init__(self, field):
        self.field = field

    def __get__(self, obj, typ=None):
        if obj is None:
            return self
        return obj.__dict__[self.field.name]        

    def __set__(self, obj, value):
        obj.__dict__[self.field.name] = self.field.to_python(value)

def make_contrib(func=None):
    """
    Returns a suitable contribute_to_class() method for the Field subclass.

    If 'func' is passed in, it is the existing contribute_to_class() method on
    the subclass and it is called before anything else. It is assumed in this
    case that the existing contribute_to_class() calls all the necessary
    superclass methods.
    """
    def contribute_to_class(self, cls, name):
        if func:
            func(self, cls, name)
        else:
            super(self.__class__, self).contribute_to_class(cls, name)
        setattr(cls, self.name, CreatorFix(self))

    return contribute_to_class



from django.utils.text import capfirst

_defined_standins = dict()

class BoolStandin(int):
    def __nonzero__(self):
        return self != 0
    def __repr__(self):
        if self == 0:
            return 'False'
        else:
            return 'True'

def standin_for(obj, **attrs):
    """
    Returns an object that can be used as a standin for the original object.
    
    The standin object will have extra attrs, which you can define passed as keyword arguments.
    
    This will be used in the __future__ to pass info about the origin of a value to
    a widget. The origin of a value can either be the database or the gettext catalog.
    You will be able to change the origin. This is very usefull if for example you
    saved a model without realising all fields will be committed to the database and
    you made changes to the gettext catalog afterwards.
    
    See :ref:`database_rules_all`
    """
    
    obj_class = obj.__class__
    if obj_class is bool:
        obj_class = BoolStandin
    
    attr_names  = attrs.keys()
    attr_names.sort()

    # create a readable class name eg. unicodeAndTitleAndDescription
    additions = 'And'.join(map(capfirst, attr_names))
    id = "%s%s" % (obj_class.__name__, additions.encode('ascii', 'ignore'))
    
    # if we allready know this type don't create it again.
    cached_type = _defined_standins.get(id, None)
    if not cached_type:
        cls_attrs = dict([(attr_name, None) for attr_name in attr_names])
        cached_type = type("%sStandIn" % id, (obj_class,), cls_attrs)
        _defined_standins[id] = cached_type
    
    # create new object based on original and add extra attrs.
    stand_in = cached_type(obj)
    for (key, value) in attrs.iteritems():
        setattr(stand_in, key, value)
    
    return stand_in


import warnings
from django.test import TestCase
from django.conf import settings
from easymode.tests.testcases import initdb
import reversion

@initdb
class TestEasyPublisher(TestCase):
    """Tests for easypublisher"""
    
    def test_it_works(self):
        pass
        
    def test_tests_are_implemented(self):
        """Test any tests are implemented for easypublisher."""
        assert("There are no tests yet" is None)

    